Enum ShouldUpdateMessages
pub enum ShouldUpdateMessages {
Always,
Waiting,
Ready,
}Expand description
Controls whether or not the messages in an MessageRegistry should be updated.
Variants
Always
Without any fixed timestep, messages should always be updated each frame.
Waiting
We need to wait until at least one pass of the fixed update schedules to update the messages.
Ready
At least one pass of the fixed update schedules has occurred, and the messages are ready to be updated.