Struct MessageCursor
pub struct MessageCursor<M>where
M: Message,{ /* private fields */ }Expand description
Stores the state for a MessageReader or MessageMutator.
Access to the Messages<M> resource is required to read any incoming messages.
In almost all cases, you should just use a MessageReader to read messages,
or a MessageMutator to modify messages or to read and write messages in the same system,
which will automatically manage the state for you.
However, this type can be useful if you need to manually track messages.
Example
use bevy_ecs::prelude::*;
use bevy_ecs::message::{Message, MessageCursor};
#[derive(Message, Clone, Debug)]
struct MyMessage;
/// A system that both sends and receives messages using a [`Local`] [`MessageCursor`].
fn send_and_receive_messages(
// The `Local` `SystemParam` stores state inside the system itself, rather than in the world.
// `MessageCursor<T>` is the internal state of `MessageMutator<T>`, which tracks which messages have been seen.
mut local_message_reader: Local<MessageCursor<MyMessage>>,
// We can access the `Messages` resource mutably, allowing us to both read and write its contents.
mut messages: ResMut<Messages<MyMessage>>,
) {
// We must collect the messages to resend, because we can't mutate messages while we're iterating over the messages.
let mut messages_to_resend = Vec::new();
for message in local_message_reader.read(&mut messages) {
messages_to_resend.push(message.clone());
}
for message in messages_to_resend {
messages.write(MyMessage);
}
}
Implementations
impl<M> MessageCursor<M>where
M: Message,
impl<M> MessageCursor<M>where
M: Message,
pub fn read<'a>(
&'a mut self,
messages: &'a Messages<M>,
) -> MessageIterator<'a, M> ⓘ
pub fn read<'a>( &'a mut self, messages: &'a Messages<M>, ) -> MessageIterator<'a, M> ⓘ
pub fn read_mut<'a>(
&'a mut self,
messages: &'a mut Messages<M>,
) -> MessageMutIterator<'a, M> ⓘ
pub fn read_mut<'a>( &'a mut self, messages: &'a mut Messages<M>, ) -> MessageMutIterator<'a, M> ⓘ
pub fn read_with_id<'a>(
&'a mut self,
messages: &'a Messages<M>,
) -> MessageIteratorWithId<'a, M> ⓘ
pub fn read_with_id<'a>( &'a mut self, messages: &'a Messages<M>, ) -> MessageIteratorWithId<'a, M> ⓘ
pub fn read_mut_with_id<'a>(
&'a mut self,
messages: &'a mut Messages<M>,
) -> MessageMutIteratorWithId<'a, M> ⓘ
pub fn read_mut_with_id<'a>( &'a mut self, messages: &'a mut Messages<M>, ) -> MessageMutIteratorWithId<'a, M> ⓘ
pub fn par_read<'a>(
&'a mut self,
messages: &'a Messages<M>,
) -> MessageParIter<'a, M>
pub fn par_read<'a>( &'a mut self, messages: &'a Messages<M>, ) -> MessageParIter<'a, M>
pub fn par_read_mut<'a>(
&'a mut self,
messages: &'a mut Messages<M>,
) -> MessageMutParIter<'a, M>
pub fn par_read_mut<'a>( &'a mut self, messages: &'a mut Messages<M>, ) -> MessageMutParIter<'a, M>
pub fn missed_messages(&self, messages: &Messages<M>) -> usize
pub fn missed_messages(&self, messages: &Messages<M>) -> usize
Amount of messages we missed.